Linux Distros That Serve Scientific and Medical Communities
============================================================

![Linux for science](https://www.linux.com/sites/lcom/files/styles/rendered_file/public/linux-science.jpg?itok=lzu8eXpt "Linux for science")
Jack Wallen looks at a few Linux distributions that specialize in serving the scientific and medical communities.[Creative Commons Zero][5]

Linux serves — of that there is no doubt — literally and figuratively. The open source platform serves up websites across the globe, it serves educational systems in numerous ways, and it also serves the medical and scientific communities and has done so for quite some time.

I remember, back in my early days of Linux usage (I first adopted Linux as my OS of choice in 1997), how every Linux distribution included so many tools I would personally never use. Tools used for plotting and calculating on levels I’d not even heard of before. I cannot remember the names of those tools, but I know they were opened once and never again. I didn’t understand their purpose. Why? Because I wasn’t knee-deep in studying such science.

Modern Linux is a far cry from those early days. Not only is it much more user-friendly, it doesn’t include that plethora of science-centric tools. There are, however, still Linux distributions for that very purpose — serving the scientific and medical communities.

Let’s take a look at a few of these distributions. Maybe one of them will suit your needs.

### Scientific Linux

You can’t start a listing of science-specific Linux distributions without first mentioning [Scientific Linux][12]. This particular take on Linux was developed by [Fermilab][13]. Based on Red Hat Enterprise Linux, Scientific Linux aims to offer a common Linux distribution for various labs and universities around the world, in order to reduce duplication of effort. The goal of Scientific Linux is to have a distribution that is compatible with Red Hat Enterprise Linux, that:

*   Provides a stable, scalable, and extensible operating system for scientific computing.

*   Supports scientific research by providing the necessary methods and procedures to enable the integration of scientific applications with the operating environment.

*   Uses the free exchange of ideas, designs, and implementations in order to prepare a computing platform for the next generation of scientific computing.

*   Includes all the necessary tools to enable users to create their own Scientific Linux spins.

Because Scientific Linux is based on Red Hat Enterprise Linux, you can select a Security Policy for the platform during installation (Figure 1).

![Scientific Linux](https://www.linux.com/sites/lcom/files/styles/rendered_file/public/scientificlinux_1.jpg?itok=Wz5I7e8i "Scientific Linux")

Figure 1: Selecting a security policy for Scientific Linux during installation.[Used with permission][1]

Two famous experiments that work with Scientific Linux are:

*   Collider Detector at Fermilab — experimental collaboration that studies high energy particle collisions at the [Tevatron][6] (a circular particle accelerator)

*   DØ experiment — a worldwide collaboration of scientists that conducts research on the fundamental nature of matter.

What you might find interesting about Scientific Linux is that it doesn’t actually include all the science-y goodness you might expect. There is no Matlab equivalent pre-installed, or other such tools. The good news is that there are plenty of repositories available that allow you to install everything you need to create a distribution that perfectly suits your needs.

Scientific Linux is available to use for free and can be downloaded from the [official download page][14].

### Bio-Linux

Now we’re venturing into territory that should make at least one cross section of scientists very happy. Bio-Linux is a distribution aimed specifically at bioinformatics (the science of collecting and analyzing complex biological data such as genetic codes). This very green-looking take on Linux (Figure 2) was developed at the [Environmental Omics Synthesis Centre ][15]and the [Natural Environment for Ecology & Hydrology][16] and includes hundreds of bioinformatics tools, including:

*   abyss — de novo, parallel, sequence assembler for short reads

*   Artemis — DNA sequence viewer and annotation tool

*   bamtools — toolkit for manipulating BAM (genome alignment) files

*   Big-blast — The big-blast script for annotation of long sequence

*   Galaxy — browser-based biomedical research platform

*   Fasta — tool for searching DNA and protein databases

*   Mesquite — used for evolutionary biology

*   njplot — tool for drawing phylogenetic trees

*   Rasmo — tool for visualizing macromolecules

![Bio-Linux](https://www.linux.com/sites/lcom/files/styles/rendered_file/public/biolinux.jpg?itok=4_Vz51f2 "Bio-Linux")

Figure 2: The Bio-Linux desktop.[Used with permission][2]

There are plenty of command line and graphical tools to be found in this niche platform. For a complete list, check out the included software page [here][17].

Bio-Linux is based on Ubuntu and is available for free download.

### Poseidon Linux

This particular Ubuntu-based Linux distribution originally started as a desktop, based on open source software, aimed at the international scientific community. Back in 2010, the platform switched directions to focus solely on bathymetry (the measurement of depth of water in oceans, seas, or lakes), seafloor mapping, GIS, and 3D visualization.

![Poseidon Linux](https://www.linux.com/sites/lcom/files/styles/floated_images/public/poseidon_4-menu.png?itok=HxqGY-jL "Poseidon Linux")

Figure 3: Poseidon Linux with menus (Image: Wikipedia).[Used with permission][3]

Poseidon Linux (Figure 3) is, effectively, Ubuntu 16.04 (complete with Ubuntu Unity, at the moment) with the addition of [GMT][25] (a collection of about 80 command-line tools for manipulating geographic and Cartesian data sets), [PROJ][26] (a standard UNIX filter function which converts geographic longitude and latitude coordinates into Cartesian coordinates), and [MB System][27] (seafloor mapping software).

Yes, Poseidon Linux is a very niche distribution, but if you need to measure the depth of water in oceans, seas, and lakes, you’ll be glad it’s available.

Download Poseidon Linux for free from the [official download site][18].

### NHSbuntu

A group of British IT specialists took on the task to tailor Ubuntu Linux to be used as a desktop distribution by the [UK National Health Service][19]. [NHSbuntu][20] was first released, as an alpha, on April 27, 2017\. The goal was to create a PC operating system that could deliver security, speed, and cost-effectiveness and to create a desktop distribution that would conform to the needs of the NHS — not insist the NHS conform to the needs of the software. NHSbuntu was set up for full disk encryption to safeguard the privacy of sensitive data.

NHSbuntu includes LibreOffice, NHSMail2 (a version of the Evolution groupware suite, capable of connecting to NHSmail2 and Trust email), and Chat (a messenger app able to work with NHSmail2). This spin on Ubuntu can:

*   Perform as a Clinical OS

*   Serve as an office desktop OS

*   Be used as in kiosk mode

*   Function as a real-time dashboard

![NHSbuntu](https://www.linux.com/sites/lcom/files/styles/floated_images/public/nshbuntu.jpg?itok=21tLaafW "NHSbuntu")

Figure 4: NHSbuntu main screen.[Used with permission][4]

The specific customizations of NHSbuntu are:

*   NHSbuntu wallpaper (Figure 4)

*   A look and feel similar to a well-known desktop

*   NHSmail2 compatibility

*   Email, calendar, address book

*   Messager, with file sharing

*   N3 VPN compatibility

*   RSA token supported

*   Removal of games

*   Inclusion of Remmina (Remote Desktop client for VDI)

NHSbuntu can be [downloaded][21], for free, for either 32- or 64-bit hardware.

### The tip of the scientific iceberg

Even if you cannot find a Linux distribution geared toward your specific branch of science or medicine, chances are you will find software perfectly capable of serving your needs. There are even organizations (such as the [Open Science Project][22] and [Neurodebian][23]) dedicated to writing and releasing open source software for the scientific community.

 _Learn more about Linux through the free ["Introduction to Linux" ][24]course from The Linux Foundation and edX._

--------------------------------------------------------------------------------

via: https://www.linux.com/learn/intro-to-linux/2017/9/linux-serves-scientific-and-medical-communities

作者：[JACK WALLEN ][a]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:https://www.linux.com/users/jlwallen
[1]:https://www.linux.com/licenses/category/used-permission
[2]:https://www.linux.com/licenses/category/used-permission
[3]:https://www.linux.com/licenses/category/used-permission
[4]:https://www.linux.com/licenses/category/used-permission
[5]:https://www.linux.com/licenses/category/creative-commons-zero
[6]:https://en.wikipedia.org/wiki/Tevatron
[7]:https://www.linux.com/files/images/scientificlinux1jpg
[8]:https://www.linux.com/files/images/biolinuxjpg
[9]:https://www.linux.com/files/images/poseidon4-menupng
[10]:https://www.linux.com/files/images/nshbuntujpg
[11]:https://www.linux.com/files/images/linux-sciencejpg
[12]:http://www.scientificlinux.org/
[13]:http://www.fnal.gov/
[14]:http://www.scientificlinux.org/downloads/
[15]:http://environmentalomics.org/omics-synthesis-centre/
[16]:https://www.environmental-research.ox.ac.uk/partners/centre-for-ecology-hydrology/
[17]:http://environmentalomics.org/bio-linux-software-list/
[18]:https://sites.google.com/site/poseidonlinux/download
[19]:http://www.nhs.uk/pages/home.aspx
[20]:https://www.nhsbuntu.org/
[21]:https://www.nhsbuntu.org/
[22]:http://openscience.org/
[23]:http://neuro.debian.net/
[24]:https://training.linuxfoundation.org/linux-courses/system-administration-training/introduction-to-linux
[25]:http://gmt.soest.hawaii.edu/
[26]:http://proj4.org/
[27]:http://svn.ilab.ldeo.columbia.edu/listing.php?repname=MB-System
